@@ -14,16 +14,17 @@
 
 import ipaddress
 import threading
+import uuid
 
 from datetime import datetime
+from gearman.constants import JOB_UNKNOWN
 from oslo.config import cfg
 from sqlalchemy import func
 
 from libra.common.api.lbaas import Device, PoolBuilding, Vip, db_session
 from libra.common.api.lbaas import Counters
-from libra.common.json_gearman import JsonJob
+from libra.common.json_gearman import JSONGearmanClient
 from libra.openstack.common import log
-from libra import gear #TODO
 
 # TODO: Lots of duplication of code here, need to cleanup
 
@@ -31,6 +32,7 @@ LOG = log.getLogger(__name__)
 
 
 class Pool(object):
+
     DELETE_SECONDS = cfg.CONF['admin_api'].delete_timer_seconds
     PROBE_SECONDS = cfg.CONF['admin_api'].probe_timer_seconds
     VIPS_SECONDS = cfg.CONF['admin_api'].vips_timer_seconds
@@ -48,8 +50,6 @@ class Pool(object):
         self.start_probe_sched()
         self.start_vips_sched()
 
-        self.gear = GearmanWork()  # set up the async gearman
-
     def shutdown(self):
         if self.probe_timer:
             self.probe_timer.cancel()
@@ -67,8 +67,9 @@ class Pool(object):
             return
         LOG.info('Running device delete check')
         try:
+            message = []
             with db_session() as session:
-                devices = session.query(Device). \
+                devices = session.query(Device).\
                     filter(Device.status == 'DELETED').all()
 
                 for device in devices:
@@ -76,12 +77,20 @@ class Pool(object):
                         'action': 'DELETE_DEVICE',
                         'name': device.name
                     }
-                    self.gear.send_delete_message(job_data)
+                    unique_uuid = str(uuid.uuid4())
+                    message.append(dict(task='libra_pool_mgm',
+                                        data=job_data,
+                                        unique=unique_uuid))
 
-                counter = session.query(Counters). \
+                counter = session.query(Counters).\
                     filter(Counters.name == 'devices_deleted').first()
                 counter.value += len(devices)
                 session.commit()
+            if not message:
+                LOG.info("No devices to delete")
+            else:
+                gear = GearmanWork()
+                gear.send_delete_message(message)
         except:
             LOG.exception("Exception when deleting devices")
 
@@ -97,8 +106,9 @@ class Pool(object):
         try:
             with db_session() as session:
                 NULL = None  # For pep8
-                vip_count = session.query(Vip). \
+                vip_count = session.query(Vip).\
                     filter(Vip.device == NULL).count()
+                LOG.info('Current available VIP pool size is: %d', vip_count)
                 if vip_count >= self.vip_pool_size:
                     LOG.info("Enough vips exist, no work to do")
                     session.commit()
@@ -123,11 +133,11 @@ class Pool(object):
         try:
             with db_session() as session:
                 # Double check we have no outstanding builds assigned to us
-                session.query(PoolBuilding). \
-                    filter(PoolBuilding.server_id == self.server_id). \
+                session.query(PoolBuilding).\
+                    filter(PoolBuilding.server_id == self.server_id).\
                     delete()
                 session.flush()
-                dev_count = session.query(Device). \
+                dev_count = session.query(Device).\
                     filter(Device.status == 'OFFLINE').count()
                 if dev_count >= self.node_pool_size:
                     LOG.info("Enough devices exist, no work to do")
@@ -159,8 +169,8 @@ class Pool(object):
             # for a long time locking tables
             self._build_nodes(build_count)
             with db_session() as session:
-                session.query(PoolBuilding). \
-                    filter(PoolBuilding.server_id == self.server_id). \
+                session.query(PoolBuilding).\
+                    filter(PoolBuilding.server_id == self.server_id).\
                     delete()
                 session.commit()
         except:
@@ -168,14 +178,30 @@ class Pool(object):
         self.start_probe_sched()
 
     def _build_nodes(self, count):
+        message = []
+        it = 0
         job_data = {'action': 'BUILD_DEVICE'}
-        for it in range(0, count):
-            self.gear.send_create_message(job_data)
+        while it < count:
+            unique_uuid = str(uuid.uuid4())
+            message.append(dict(task='libra_pool_mgm',
+                                data=job_data,
+                                unique=unique_uuid))
+            it += 1
+        gear = GearmanWork()
+        gear.send_create_message(message)
 
     def _build_vips(self, count):
+        message = []
+        it = 0
         job_data = {'action': 'BUILD_IP'}
-        for it in range(0, count):
-            self.gear.send_vips_message(job_data)
+        while it < count:
+            unique_uuid = str(uuid.uuid4())
+            message.append(dict(task='libra_pool_mgm',
+                                data=job_data,
+                                unique=unique_uuid))
+            it += 1
+        gear = GearmanWork()
+        gear.send_vips_message(message)
 
     def start_probe_sched(self):
         seconds = datetime.now().second
@@ -213,126 +239,160 @@ class Pool(object):
 
 class GearmanWork(object):
 
-    class VIPClient(gear.Client):
-        def handleWorkComplete(self, packet):
-            job = super(GearmanWork.VIPClient, self).handleWorkComplete(packet)
+    def __init__(self):
+        server_list = []
+        for server in cfg.CONF['gearman']['servers']:
+            host, port = server.split(':')
+            server_list.append({'host': host,
+                                'port': int(port),
+                                'keyfile': cfg.CONF['gearman']['ssl_key'],
+                                'certfile': cfg.CONF['gearman']['ssl_cert'],
+                                'ca_certs': cfg.CONF['gearman']['ssl_ca'],
+                                'keepalive': cfg.CONF['gearman']['keepalive'],
+                                'keepcnt': cfg.CONF['gearman']['keepcnt'],
+                                'keepidle': cfg.CONF['gearman']['keepidle'],
+                                'keepintvl': cfg.CONF['gearman']['keepintvl']
+                                })
+        self.gearman_client = JSONGearmanClient(server_list)
+
+    def send_delete_message(self, message):
+        LOG.info("Sending %d gearman messages", len(message))
+        job_status = self.gearman_client.submit_multiple_jobs(
+            message, background=False, wait_until_complete=True,
+            max_retries=10, poll_timeout=30.0
+        )
+        delete_count = 0
+        for status in job_status:
+            if status.state == JOB_UNKNOWN:
+                LOG.error('Gearman Job server fail')
+                continue
+            if status.timed_out:
+                LOG.error('Gearman timeout whilst deleting device')
+                continue
+            if status.result['response'] == 'FAIL':
+                LOG.error(
+                    'Pool manager failed to delete a device, removing from DB'
+                )
+
+            delete_count += 1
+            with db_session() as session:
+                session.query(Device).\
+                    filter(Device.name == status.result['name']).delete()
+                session.commit()
+
+        LOG.info('%d freed devices delete from pool', delete_count)
+
+    def send_vips_message(self, message):
+        # TODO: make this gearman part more async, not wait for all builds
+        LOG.info("Sending %d gearman messages", len(message))
+        job_status = self.gearman_client.submit_multiple_jobs(
+            message, background=False, wait_until_complete=True,
+            max_retries=10, poll_timeout=3600.0
+        )
+        built_count = 0
+        for status in job_status:
+            if status.state == JOB_UNKNOWN:
+                LOG.error('Gearman Job server fail')
+                continue
+            if status.timed_out:
+                LOG.error('Gearman timeout whilst building vip')
+                continue
+            if status.result['response'] == 'FAIL':
+                LOG.error('Pool manager failed to build a vip')
+                continue
+
+            built_count += 1
             try:
-                if job.msg['response'] == 'FAIL':
-                    LOG.error('Pool manager failed to build a vip')
-                else:
-                    self._add_vip(job.msg)
+                self._add_vip(status.result)
             except:
                 LOG.exception(
                     'Could not add vip to DB, node data: {0}'
-                    .format(job.msg)
+                    .format(status.result)
                 )
+        LOG.info(
+            '{vips} vips built and added to pool'.format(vips=built_count)
+        )
 
-        def _add_vip(self, data):
-            LOG.info('Adding vip {0} to DB'.format(data['ip']))
-            vip = Vip()
-            vip.ip = int(ipaddress.IPv4Address(unicode(data['ip'])))
-            with db_session() as session:
-                session.add(vip)
-                counter = session.query(Counters). \
-                    filter(Counters.name == 'vips_built').first()
-                counter.value += 1
-                session.commit()
+    def send_create_message(self, message):
+        # TODO: make this gearman part more async, not wait for all builds
+        LOG.info("Sending {0} gearman messages".format(len(message)))
+        job_status = self.gearman_client.submit_multiple_jobs(
+            message, background=False, wait_until_complete=True,
+            max_retries=10, poll_timeout=3600.0
+        )
+        built_count = 0
+        for status in job_status:
+            if status.state == JOB_UNKNOWN:
+                LOG.error('Gearman Job server fail')
+                continue
+            if status.timed_out:
+                LOG.error('Gearman timeout whilst building device')
+                continue
+            if status.result['response'] == 'FAIL':
+                LOG.error('Pool manager failed to build a device')
+                if 'name' in status.result:
+                    self._add_bad_node(status.result)
+                continue
 
-    class DeleteClient(gear.Client):
-        def handleWorkComplete(self, packet):
-            job = super(GearmanWork.DeleteClient,
-                        self).handleWorkComplete(packet)
-
-            if job.msg['response'] == 'FAIL':
-                LOG.error(
-                    'Pool manager failed to delete a device, removing from DB')
-
-            self._delete_from_db(job.msg)
-
-        def _delete_from_db(self, msg):
-            with db_session() as session:
-                session.query(Device). \
-                    filter(Device.name == msg['name']).delete()
-                session.commit()
-                LOG.info("Delete device %s" % msg['name'])
-
-    class CreateClient(gear.Client):
-        def handleWorkComplete(self, packet):
-            job = super(GearmanWork.CreateClient,
-                        self).handleWorkComplete(packet)
+            built_count += 1
             try:
-                if job.msg['response'] == 'FAIL':
-                    LOG.error('Pool manager failed to build a device')
-                    if 'name' in job.msg:
-                        self._add_bad_node(job.msg)
-                else:
-                    self._add_node(job.msg)
+                self._add_node(status.result)
             except:
                 LOG.exception(
                     'Could not add node to DB, node data: {0}'
-                    .format(job.msg)
+                    .format(status.result)
                 )
+        LOG.info(
+            '{nodes} devices built and added to pool'.format(nodes=built_count)
+        )
 
-        def _add_node(self, data):
-            LOG.info('Adding device {0} to DB'.format(data['name']))
-            device = Device()
-            device.name = data['name']
-            device.publicIpAddr = data['addr']
-            # TODO: kill this field, make things use publicIpAddr instead
-            device.floatingIpAddr = data['addr']
-            device.az = data['az']
-            device.type = data['type']
-            device.pingCount = 0
-            device.status = 'OFFLINE'
-            device.created = None
-            with db_session() as session:
-                session.add(device)
-                counter = session.query(Counters). \
-                    filter(Counters.name == 'devices_built').first()
-                counter.value += 1
-                session.commit()
+    def _add_vip(self, data):
+        LOG.info('Adding vip {0} to DB'.format(data['ip']))
+        vip = Vip()
+        vip.ip = int(ipaddress.IPv4Address(unicode(data['ip'])))
+        with db_session() as session:
+            session.add(vip)
+            counter = session.query(Counters).\
+                filter(Counters.name == 'vips_built').first()
+            counter.value += 1
+            session.commit()
 
-        def _add_bad_node(self, data):
-            LOG.info(
-                "Adding bad device {0} to DB to be deleted" % (data['name']))
-            device = Device()
-            device.name = data['name']
-            device.publicIpAddr = data['addr']
-            # TODO: kill this field, make things use publicIpAddr instead
-            device.floatingIpAddr = data['addr']
-            device.az = data['az']
-            device.type = data['type']
-            device.pingCount = 0
-            device.status = 'DELETED'
-            device.created = None
-            with db_session() as session:
-                session.add(device)
-                counter = session.query(Counters). \
-                    filter(Counters.name == 'devices_bad_built').first()
-                counter.value += 1
-                session.commit()
+    def _add_node(self, data):
+        LOG.info('Adding device {0} to DB'.format(data['name']))
+        device = Device()
+        device.name = data['name']
+        device.publicIpAddr = data['addr']
+        # TODO: kill this field, make things use publicIpAddr instead
+        device.floatingIpAddr = data['addr']
+        device.az = data['az']
+        device.type = data['type']
+        device.pingCount = 0
+        device.status = 'OFFLINE'
+        device.created = None
+        with db_session() as session:
+            session.add(device)
+            counter = session.query(Counters).\
+                filter(Counters.name == 'devices_built').first()
+            counter.value += 1
+            session.commit()
 
-    def __init__(self):
-        self.vip_client = GearmanWork.VIPClient("Vip Client")
-        self.delete_client = GearmanWork.DeleteClient("Delete Client")
-        self.create_client = GearmanWork.CreateClient("Create Client")
-
-        for x in [self.vip_client, self.create_client, self.delete_client]:
-            self._init_client(x)
-
-    def _init_client(self, client):
-        client.log = LOG
-        for server in cfg.CONF['gearman']['servers']:
-            host, port = server.split(':')
-            client.addServer(host, port, cfg.CONF['gearman']['ssl_key'],
-                             cfg.CONF['gearman']['ssl_cert'],
-                             cfg.CONF['gearman']['ssl_ca'])
-
-    def send_delete_message(self, message, name='libra_pool_mgm'):
-        self.delete_client.submitJob(JsonJob(name, message))
-
-    def send_vips_message(self, message, name='libra_pool_mgm'):
-        self.vip_client.submitJob(JsonJob(name, message))
-
-    def send_create_message(self, message, name='libra_pool_mgm'):
-        self.create_client.submitJob(JsonJob(name, message))
+    def _add_bad_node(self, data):
+        LOG.info(
+            'Adding bad device {0} to DB to be deleted'.format(data['name'])
+        )
+        device = Device()
+        device.name = data['name']
+        device.publicIpAddr = data['addr']
+        # TODO: kill this field, make things use publicIpAddr instead
+        device.floatingIpAddr = data['addr']
+        device.az = data['az']
+        device.type = data['type']
+        device.pingCount = 0
+        device.status = 'DELETED'
+        device.created = None
+        with db_session() as session:
+            session.add(device)
+            counter = session.query(Counters).\
+                filter(Counters.name == 'devices_bad_built').first()
+            counter.value += 1
+            session.commit()
