@@ -18,15 +18,21 @@ from pecan.rest import RestController
 import wsmeext.pecan as wsme_pecan
 from wsme.exc import ClientSideError
 from wsme import Unset
+from urllib import quote
 from libra.common.api.lbaas import LoadBalancer, db_session
-from libra.common.api.lbaas import Device, HealthMonitor
+from libra.common.api.lbaas import Device, HealthMonitor, Counters
 from libra.api.acl import get_limited_to_project
 from libra.api.model.validators import LBMonitorPut, LBMonitorResp
 from libra.common.api.gearman_client import submit_job
-from libra.api.library.exp import NotFound
+from libra.api.library.exp import NotFound, ImmutableEntity, ImmutableStates
 
 
 class HealthMonitorController(RestController):
+
+    TIMEOUT_LIMIT = 3600
+    DELAY_LIMIT = 3600
+    PATH_LIMIT = 2000
+
     """functions for /loadbalancers/{loadBalancerId}/healthmonitor routing"""
     def __init__(self, load_balancer_id=None):
         self.lbid = load_balancer_id
@@ -70,7 +76,14 @@ class HealthMonitorController(RestController):
             if monitor.path:
                 monitor_data['path'] = monitor.path
 
-            session.commit()
+            try:
+                counter = session.query(Counters).\
+                    filter(Counters.name == 'api_healthmonitor_get').first()
+                counter.value += 1
+
+                session.commit()
+            except:
+                pass
         return monitor_data
 
     @wsme_pecan.wsexpose(LBMonitorResp, body=LBMonitorPut, status_code=202)
@@ -110,9 +123,13 @@ class HealthMonitorController(RestController):
             # Check inputs
             if (
                 body.type == Unset or
+                body.type is None or
                 body.delay == Unset or
+                body.delay is None or
                 body.timeout == Unset or
-                body.attemptsBeforeDeactivation == Unset
+                body.timeout is None or
+                body.attemptsBeforeDeactivation == Unset or
+                body.attemptsBeforeDeactivation is None
             ):
                 session.rollback()
                 raise ClientSideError(
@@ -136,13 +153,23 @@ class HealthMonitorController(RestController):
                     raise ClientSideError(
                         "Path argument is invalid with CONNECT type"
                     )
-                data["path"] = body.path
+                # Encode everything apart from allowed characters
+                # This ignore list in the second parameter is everything in
+                # RFC3986 section 2 that isn't already ignored by
+                # urllib.quote()
+                data["path"] = quote(body.path, "/~+*,;:!$'[]()?&=#%")
                 # If path is empty, set to /
                 if len(data["path"]) == 0 or data["path"][0] != "/":
                     session.rollback()
                     raise ClientSideError(
                         "Path must begin with leading /"
                     )
+
+                if len(data["path"]) > self.PATH_LIMIT:
+                    raise ClientSideError(
+                        "Path must be less than {0} characters"
+                        .format(self.PATH_LIMIT)
+                    )
             else:
                 if body.type != "CONNECT":
                     session.rollback()
@@ -151,6 +178,22 @@ class HealthMonitorController(RestController):
                     )
                 data["path"] = None
 
+            # Check timeout limits. Must be > 0 and limited to 1 hour
+            if data["timeout"] < 1 or data["timeout"] > self.TIMEOUT_LIMIT:
+                session.rollback()
+                raise ClientSideError(
+                    "timeout must be between 1 and {0} seconds"
+                    .format(self.TIMEOUT_LIMIT)
+                )
+
+            # Check delay limits. Must be > 0 and limited to 1 hour
+            if data["delay"] < 1 or data["delay"] > self.DELAY_LIMIT:
+                session.rollback()
+                raise ClientSideError(
+                    "delay must be between 1 and {0} seconds"
+                    .format(self.DELAY_LIMIT)
+                )
+
             if data["timeout"] > data["delay"]:
                 session.rollback()
                 raise ClientSideError(
@@ -180,6 +223,14 @@ class HealthMonitorController(RestController):
                 monitor.attempts = data["attempts"]
                 monitor.path = data["path"]
 
+            if lb.status in ImmutableStates:
+                session.rollback()
+                raise ImmutableEntity(
+                    'Cannot modify a Load Balancer in a non-ACTIVE state'
+                    ', current state: {0}'
+                    .format(lb.status)
+                )
+
             lb.status = 'PENDING_UPDATE'
             device = session.query(
                 Device.id, Device.name, Device.status
@@ -187,12 +238,6 @@ class HealthMonitorController(RestController):
                 filter(LoadBalancer.id == self.lbid).\
                 first()
 
-            if device.status == 'ERROR':
-                session.rollback()
-                raise ClientSideError(
-                    'Cannot modify a Load Balancer in an ERROR state'
-                )
-
             return_data = LBMonitorResp()
             return_data.type = data["type"]
             return_data.delay = str(data["delay"])
@@ -202,7 +247,13 @@ class HealthMonitorController(RestController):
             if ((data["path"] is not None) and (len(data["path"]) > 0)):
                 return_data.path = data["path"]
 
-            session.commit()
+            try:
+                counter = session.query(Counters).\
+                    filter(Counters.name == 'api_healthmonitor_modify').first()
+                counter.value += 1
+                session.commit()
+            except:
+                pass
             submit_job(
                 'UPDATE', device.name, device.id, lb.id
             )
@@ -251,7 +302,13 @@ class HealthMonitorController(RestController):
             ).join(LoadBalancer.devices).\
                 filter(LoadBalancer.id == self.lbid).\
                 first()
-            session.commit()
+            try:
+                counter = session.query(Counters).\
+                    filter(Counters.name == 'api_healthmonitor.delete').first()
+                counter.value += 1
+                session.commit()
+            except:
+                pass
             submit_job(
                 'UPDATE', device.name, device.id, self.lbid
             )
